#if defined(TRANSPARENT)
varying vec4 v_position;
uniform int u_outputIndex;
float computeWeight(vec4 color)
{
    // Insert your favorite weighting function here. The color-based factor
    // avoids color pollution from the edges of wispy clouds. The z-based
    // factor gives precedence to nearer surfaces.
    float weight = max(min(1.0, max(max(color.r, color.g), color.b) * color.a), color.a) * clamp(0.03 / (1e-5 + pow(v_position.z / 200.0, 4.0)), 1e-2, 3e3);
    return weight;
}

void calcOutput(vec4 color) {
    float weight = computeWeight(color);
    // 这里需要*color.a以变成premultiplied
    vec4 accumColor = vec4(color.rgb * color.a * weight, color.a);
    vec4 revealColor = vec4(color.a * weight);
    gl_FragColor = vec4(1 - u_outputIndex) * accumColor + vec4(u_outputIndex) * revealColor;
}
#else

#ifdef OPENGL_ES
#ifdef GL_FRAGMENT_PRECISION_HIGH
precision highp float;
#else
precision mediump float;
#endif
#endif

#ifndef PASSTHROUGH
uniform sampler2D u_accumTexture;
uniform sampler2D u_revealageTexture;
varying vec2 v_texCoord;

void main(void)
{
    vec4 accum = texture2D(u_accumTexture, v_texCoord);
    float r = accum.a;
    accum.a = texture2D(u_revealageTexture, v_texCoord).r;
    gl_FragColor = vec4(accum.rgb / clamp(accum.a, 1e-4, 5e4), 1.0 - r);
}
#else
uniform sampler2D u_texture;
varying vec2 v_texCoord;

void main(void)
{
    gl_FragColor = texture2D(u_texture, v_texCoord);
}
#endif
#endif
