#ifndef DIRECTIONAL_LIGHT_COUNT
#define DIRECTIONAL_LIGHT_COUNT 0
#endif
#ifndef SPOT_LIGHT_COUNT
#define SPOT_LIGHT_COUNT 0
#endif
#ifndef POINT_LIGHT_COUNT
#define POINT_LIGHT_COUNT 0
#endif
#if (DIRECTIONAL_LIGHT_COUNT > 0) || (POINT_LIGHT_COUNT > 0) || (SPOT_LIGHT_COUNT > 0)
#define LIGHTING
#endif
#define TEXTURED

///////////////////////////////////////////////////////////
// Atributes
attribute vec4 a_position;

#if defined(SKINNING)
attribute vec4 a_blendWeights;
attribute vec4 a_blendIndices;
#endif

attribute vec2 a_texCoord;

#if defined(LIGHTMAP)
attribute vec2 a_texCoord1; 
#endif

#if defined(LIGHTING)
attribute vec3 a_normal;

#if defined(BUMPED)
attribute vec3 a_tangent;
attribute vec3 a_binormal;
#endif

#endif

///////////////////////////////////////////////////////////
// Uniforms
uniform mat4 u_worldViewProjectionMatrix;
#if defined(SKINNING)
uniform vec4 u_matrixPalette[SKINNING_JOINT_COUNT * 3];
#endif

#if defined(LIGHTING)
uniform mat4 u_inverseTransposeWorldMatrix;

#if defined(SPECULAR) || (POINT_LIGHT_COUNT > 0) || (SPOT_LIGHT_COUNT > 0) || defined(CLIP_PLANE) || defined(SHADOW_MAPPING) || defined(PBR)
uniform mat4 u_worldMatrix;
#endif

#if (defined(BUMPED) || defined(PBR)) && (DIRECTIONAL_LIGHT_COUNT > 0)
uniform vec3 u_directionalLightDirection[DIRECTIONAL_LIGHT_COUNT];
#endif

#if (POINT_LIGHT_COUNT > 0)
uniform vec3 u_pointLightPosition[POINT_LIGHT_COUNT];
#endif

#if (SPOT_LIGHT_COUNT > 0) 
uniform vec3 u_spotLightPosition[SPOT_LIGHT_COUNT];
#if defined(BUMPED) || defined(PBR)
uniform vec3 u_spotLightDirection[SPOT_LIGHT_COUNT];
#endif
#endif

#if defined(SPECULAR) || defined(PBR)
uniform vec3 u_cameraPosition;
#endif

#endif

#if defined(TEXTURE_REPEAT)
uniform vec2 u_textureRepeat;
#endif

#if defined(TEXTURE_SHEET)
uniform vec4 u_sheetProperty;
uniform vec2 u_sheetIndex;
#elif defined(TEXTURE_OFFSET)
uniform vec2 u_textureOffset;
#endif

#if defined(CLIP_PLANE)
uniform vec4 u_clipPlane;
#endif

#if defined(SHADOW_MAPPING)
uniform mat4 u_shadowTextureMatrix;
uniform vec2 u_depthRange; // x: nearPlane; y: 1.0 / (farPlane - nearPlane);
#endif

///////////////////////////////////////////////////////////
// Varyings
varying vec2 v_texCoord;

#if defined(LIGHTMAP)
varying vec2 v_texCoord1;
#endif

#if defined(LIGHTING)

#if !defined(BUMPED)
varying vec3 v_normalVector;
#endif

#if (defined(BUMPED) || defined(PBR)) && (DIRECTIONAL_LIGHT_COUNT > 0)
varying vec3 v_directionalLightDirection[DIRECTIONAL_LIGHT_COUNT];
#endif

#if (POINT_LIGHT_COUNT > 0)
varying vec3 v_vertexToPointLightDirection[POINT_LIGHT_COUNT];
#endif

#if (SPOT_LIGHT_COUNT > 0)
varying vec3 v_vertexToSpotLightDirection[SPOT_LIGHT_COUNT];
#if defined(BUMPED) || defined(PBR)
varying vec3 v_spotLightDirection[SPOT_LIGHT_COUNT];
#endif
#endif

#if defined(SPECULAR) || defined(PBR)
varying vec3 v_cameraDirection;
#endif

#if defined(BUMPED) && defined(USE_IBL)
varying vec3 v_cameraDirectionWorldSpace;
varying mat3 v_tangentToWorldSpaceTransformMatrix;
#endif

#if defined(ENVIRONMENT_TEXTURING)
uniform mat4 u_worldViewProjectionValue;
uniform float u_updateEnvironmentTexture;
uniform float u_environmentEnable;
#endif

#include "lighting.vert"

#endif

#if defined(SKINNING)
#include "skinning.vert"
#else
#include "skinning-none.vert" 
#endif

#if defined(CLIP_PLANE)
varying float v_clipDistance;
#endif

#if defined(SHADOW_MAPPING)
varying vec4 v_shadowTexCoord;
#endif

#if defined(TRANSPARENT)
varying vec4 v_position;
#endif

void main()
{
    vec4 position = getPosition();
    gl_Position = u_worldViewProjectionMatrix * position;
#if defined(TRANSPARENT)
    v_position = gl_Position;
#endif

    #if defined(LIGHTING)
    vec3 normal = getNormal();
    // 这里的变换比较特殊，因为法线可能会由于缩放比例不一样而导致法线与平面不垂直，
    // 因为不能直接乘以WorldMatrix或者WorldViewMatrix，而是要乘以一个叫做法线
    // 矩阵的矩阵，参考：
    // http://www.lighthouse3d.com/tutorials/glsl-12-tutorial/the-normal-matrix/
    // Transform the normal, tangent and binormals to world space.
    mat3 inverseTransposeWorldMatrix = mat3(u_inverseTransposeWorldMatrix[0].xyz, u_inverseTransposeWorldMatrix[1].xyz, u_inverseTransposeWorldMatrix[2].xyz);
    vec3 normalVector = normalize(inverseTransposeWorldMatrix * normal);
    
    #if defined(BUMPED)
    
    vec3 tangent = getTangent();
    vec3 binormal = getBinormal();
    vec3 tangentVector  = normalize(inverseTransposeWorldMatrix * tangent);
    vec3 binormalVector = normalize(inverseTransposeWorldMatrix * binormal);
    mat3 tangentSpaceTransformMatrix = mat3(tangentVector.x, binormalVector.x, normalVector.x, tangentVector.y, binormalVector.y, normalVector.y, tangentVector.z, binormalVector.z, normalVector.z);
    applyLight(position, tangentSpaceTransformMatrix);
    
    #if defined(USE_IBL)
    v_tangentToWorldSpaceTransformMatrix = mat3(tangentVector, binormalVector, normalVector);
    #endif
    
    #else
    
    v_normalVector = normalVector;
    applyLight(position);
    
    #endif
    
    #endif

    #if defined(ENVIRONMENT_TEXTURING)
    //是否更新，需要更新则每次都使用新的gl_position，不更新则使用外部传入的uniform值（外部获取节点的MVP）
    if (u_environmentEnable > 0.0) {
        vec4 environmentPosition;
        //由于position受骨骼动画影响会变,导致停止更新时的顶点信息依旧更新，所以这里当停止环境纹理更新时，使用骨骼动画作用之前的顶点坐标信息
        //需要更新时u_updateEnvironmentTexture为1.f，否则为0.f
        environmentPosition = (1.0 - u_updateEnvironmentTexture) * u_worldViewProjectionValue * a_position
        + u_updateEnvironmentTexture * u_worldViewProjectionMatrix * position;
        //使用gl_position转变坐标系到UV坐标系下(做透视划分之后的值)作为纹理坐标
        v_texCoord = vec2(environmentPosition.x/environmentPosition.w/2.0 + 0.5, environmentPosition.y/environmentPosition.w/2.0 + 0.5);
    }else{
        v_texCoord = a_texCoord;
    }
    #else
    v_texCoord = a_texCoord;
    #endif
    
    #if defined(TEXTURE_REPEAT)
    v_texCoord *= u_textureRepeat;
    #endif

    #if defined(TEXTURE_SHEET)
    float col = u_sheetProperty.x;
    float row = u_sheetProperty.y;
    if(col >= 1.0 && row >= 1.0){
        float index = mod(u_sheetIndex.x + 1.0, u_sheetProperty.z);
        float colIndex = mod(index - 1.0, col);
        index = mod(u_sheetIndex.x, u_sheetProperty.z);
        float rowIndex = floor(index / col) ;
        v_texCoord.x = (v_texCoord.x + colIndex) / col;
        v_texCoord.y = (v_texCoord.y + row - rowIndex - 1.0) / row;
    }
    #elif defined(TEXTURE_OFFSET)
    v_texCoord += u_textureOffset;
    #endif
    
    #if defined(LIGHTMAP)
    v_texCoord1 = a_texCoord1;
    #endif
    
    #if defined(CLIP_PLANE)
    v_clipDistance = dot(u_worldMatrix * position, u_clipPlane);
    #endif
    
    #if defined(SHADOW_MAPPING)
    vec4 worldPos      = u_worldMatrix * position;
    //将世界坐标系下的点，变换到阴影正交相机中；同时求改阴影平面的在shadowmap中的uv坐标,可超出（0，1）
    v_shadowTexCoord   = u_shadowTextureMatrix * worldPos;
    v_shadowTexCoord.z = (v_shadowTexCoord.z - u_depthRange.x) * u_depthRange.y;
    #endif
}
